% ==========================================================
% 
%           Neural Networks A Classroom Approach
%                     Satish Kumar
%             Copyright Tata McGraw Hill, 2004
%
%       MATLAB code that implements the Boltzmann 
%       learning algorithm 
%             Reference: Table 10.9;Page 437
%
% ==========================================================

%Augmented desired vectors
A=[1 0];
Ab=2*A-1;

rand('state',sum(100*clock));
W = (2 * rand(4,3) - 1) .* [(1-eye(3)); ones(1,3)]; % remove diagonal, last row is bias

S=[0 0 0
   0 0 1
   0 1 0
   0 1 1
   1 0 0
   1 0 1
   1 1 0
   1 1 1];
Sb = 2*S - 1;

c = 0.99;
eta = 0.002;
rep_num = 600;
num_cycles = 300;

for k = 1:100
   rand('state',sum(100*clock));
   free_signals = [(2*round(rand(rep_num, 3)) - 1) -ones(rep_num,1) ]; %append bias signals
   clamped_h_signals = [(2*round(rand(rep_num,1)) - 1) -ones(rep_num,1) ];
   
   T = 10;
   while (T > 1)
      for relax = 1:num_cycles 
         rand('state',sum(100*clock));
         Xf = free_signals * W;			% compute activations
         pf = 1./(1 + exp(-Xf./T));		% compute the flip probabilities
         randf = rand(rep_num, 3);		% generate random #s in (0,1)
         
         for i = 1:rep_num      			% work down network instances 
            j = round(1+2*rand);
            if (randf(i,j) < pf(i,j)) 
               free_signals(i,j) = 1;
            else free_signals(i,j) = -1;
            end
         end
         
         rand('state',sum(100*clock));
         clamped_signals = [repmat(Ab, [rep_num 1]) clamped_h_signals];
         Xc = clamped_signals * W;
         pc = 1./(1 + exp(-Xc./T));	% compute the flip probabilities
         randc = rand(rep_num, 3);  % generate random #s in (0,1)
         
         for i=1:rep_num
            j = round(1+2*rand); 
            if (randc(i,j) < pc(i,j)) 
               clamped_signals(i,j) = 1;
            else clamped_signals(i,j) = -1;
            end
         end
         clamped_h_signals = clamped_signals(:,3:4);
      end
      T = T*c
   end
   clamped_signals = [repmat(Ab, [rep_num 1]) clamped_h_signals];
   
   %Collect stats and change weights
   Expc = (1/rep_num)*clamped_signals' * clamped_signals
   Expf = (1/rep_num)*free_signals'* free_signals
   deltaW = Expc(:,1:3) - Expf(:,1:3);
   W = W + (eta/T) * deltaW.*  [(1-eye(3)); ones(1,3)]
end